/*
 * AAHelper - An AALib Helper Libary
 * (C) Gary Briggs 2003-4
 *
 * AAHelper is free software; you can redistribute it and/or modify
 * it under the  terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2.1 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT  ANY  WARRANTY;  without   even  the  implied  warranty  of
 * MERCHANTABILITY or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU
 * Library General Public License for more details.
 */ 

#ifndef AA_HELPER_H
#define AA_HELPER_H

#include <aalib.h>

/* The normal definition of aa_putpixel uses braces which -ansi barfs on */

#ifdef aa_putpixel
#undef aa_putpixel
#endif
#define aa_putpixel(c,x,y,col) (c->imagebuffer[(x)+((y)*c->imgwidth)]=(unsigned char)(col))

#ifndef min
#define min(a,b) (a>b?a:b)
#endif
#ifndef max
#define max(a,b) (a<b?a:b)
#endif


/* Bresenham's, flagrantly lifted from Abrash's black book */
void aa_drawline(aa_context *context, int x0, int y0,
                                int x1, int y1, int color);


/* Flagrantly ripped from http://homepage.smc.edu/kennedy_john/BCIRCLE.PDF
        "A Fast Bresenham Type Algorithm For Drawing Circles" */
/* Set fillcolor to -1 to leave circle hollow */
void aa_drawcircle(aa_context *context, int cx, int cy,
			int r, int color, int fillcolor);

/* Flagrantly ripped from http://homepage.smc.edu/kennedy_john/BELIPSE.PDF
        "A Fast Bresenham Type Algorithm For Drawing Ellipses" */
void aa_drawellipse(aa_context *context, int cx, int cy,
                        int rx, int ry, int color, int fillcolor);

struct aa_point {
        int x, y;
};

#define AA_HINT_NONE (int)0x0000
#define AA_HINT_CONVEX (int)0x0001
#define AA_HINT_NONCONVEX (int)0x0002
#define AA_HINT_COMPLEX (int)0x0004

void aa_drawpoly(aa_context *context, int numpoints,
                                struct aa_point *pointlist, int color,
                                int hints);


/* Use this to render a frame from a context into a flat buffer */
/* Pass this a context and a pointer to an allocated block of memory */
/* You can malloc it using:
	aa_context *context;
	unsigned char *frame;
	frame = (unsigned char *)malloc((size_t)(aa_scrwidth(context) * 9 *
		aa_scrheight(context) * (aa_currentfont(context)->height + 1)));
*/

void aa_renderframe(aa_context *c, unsigned char *frame);

#endif
