/*
 * AAHelper - An AALib Helper Libary
 * (C) Gary Briggs 2003-4
 *
 * AAHelper is free software; you can redistribute it and/or modify
 * it under the  terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2.1 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT  ANY  WARRANTY;  without   even  the  implied  warranty  of
 * MERCHANTABILITY or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU
 * Library General Public License for more details.
 */ 


#ifndef AA_HELPERINT_H
#define AA_HELPERINT_H

#include <aalib.h>
#include "aa_helper.h"

/* Functions for internal use */

/*
 * Draws 8 of the points on the circle radius
 * For internal use, Use aa_drawcircle instead.
 */
void aa_plot8circlepoints(aa_context *context, int x, int y,
			int cx, int cy, int color, int fillcolor);

/*
 * Draws 4 of the points on the ellipse
 * For internal use, Use aa_drawellipse instead.
 */
void aa_plot4ellipsepoints(aa_context *context, int x, int y,
                        int cx, int cy, int color, int fillcolor);

/*
 * Draws a line in octant 0 or 3 ( |xlen| >= ylen ).
 * For internal use, Use aa_drawline instead.
 */
void octant0(aa_context *context, int x0, int y0,
			int xlen, int ylen,
                        int xdir, int color);

/*
 * Draws a line in octant 1 or 2 ( |xlen| < ylen ).
 * For internal use, Use aa_drawline instead.
 */
void octant1(aa_context *context, int x0, int y0,
			int xlen, int ylen,
                        int xdir, int color);

/*
 * Draws a convex poly
 * For internal use, Use aa_drawpoly instead.
 */
void aa_drawconvexpoly(aa_context *context, int numpoints,
                                struct aa_point *pointlist, int color);

/*
 * Draws a nonconvex poly
 * For internal use, Use aa_drawpoly instead.
 */
void aa_drawnonconvexpoly(aa_context *context, int numpoints,
                                struct aa_point *pointlist, int color);

/*
 * Draws a complex poly
 * For internal use, Use aa_drawpoly instead.
 */
void aa_drawcomplexpoly(aa_context *context, int numpoints,
                                struct aa_point *pointlist, int color);

/* Scan a list for aa_drawline */
void aa_scanedge(int x0, int y0, int x1, int y1, int *pointlist);

/* Draw some stuff into a buffer */
void aa_tmpbuff_bres(int x0, int y0, int x1, int y1, int w, char *buffer);
void aa_tmpbuff_octant0(int x0, int y0,
                        int xlen, int ylen,
                        int w, int xdir, char *buffer);
void aa_tmpbuff_octant1(int x0, int y0,
                        int xlen, int ylen,
                        int w, int xdir, char *buffer);

/* Used to render an individual frame; get the font stuff */
unsigned char getfontpixel(__AA_CONST struct aa_font *font, unsigned char c,
	unsigned char x, unsigned char y);


struct aa_EdgeState;
struct aa_EdgeState {
   struct aa_EdgeState *NextEdge;
   int X;
   int StartY;
   int WholePixelXMove;
   int XDirection;
   int ErrorTerm;
   int ErrorTermAdjUp;
   int ErrorTermAdjDown;
   int Count;
};

void aa_BuildGET(struct aa_point *VertexList, int numvertices,
	struct aa_EdgeState *NextFreeEdgeStruc, int XOffset, int YOffset,
	struct aa_EdgeState **aa_GETPtr);
void aa_MoveXSortedToAET(int YToMove, struct aa_EdgeState **aa_GETPtr,
	struct aa_EdgeState **aa_AETPtr);
void aa_ScanOutAET(aa_context *c, int YToScan, int color,
	struct aa_EdgeState **aa_AETPtr);
void aa_AdvanceAET(struct aa_EdgeState **aa_AETPtr);
void aa_XSortAET(struct aa_EdgeState **aa_AETPtr);

/* struct aa_EdgeState *aa_GETPtr, *aa_AETPtr; */
#ifndef SWAP
#define SWAP(a,b) {int t; t=a; a=b; b=t;}
#endif

#endif
