/*
 * AAHelper - An AALib Helper Libary
 * (C) Gary Briggs 2003-4
 *
 * AAHelper is free software; you can redistribute it and/or modify
 * it under the  terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2.1 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT  ANY  WARRANTY;  without   even  the  implied  warranty  of
 * MERCHANTABILITY or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU
 * Library General Public License for more details.
 */ 

#include <aalib.h>
#include <math.h>
#include "./aa_helper.h"
#include "./aa_helperint.h"


/* Flagrantly ripped from http://homepage.smc.edu/kennedy_john/BELIPSE.PDF
        "A Fast Bresenham Type Algorithm For Drawing Ellipses" */
void aa_drawellipse(aa_context *context, int cx, int cy,
                        int rx, int ry, int color, int fillcolor) {
	int x, y;
	int xchange, ychange;
	int ellipseerror;
	int twoasquare, twobsquare;
	int stoppingx, stoppingy;

	twoasquare = 2 * rx * rx;
	twobsquare = 2 * ry * ry;
	x = rx;
	y = 0;
	xchange = ry * ry * (1 - 2 * rx);
	ychange = rx * rx;
	ellipseerror = 0;
	stoppingx = twobsquare * rx;
	stoppingy = 0;

	while (stoppingx >= stoppingy) {
		aa_plot4ellipsepoints(context, x, y, cx, cy, color, fillcolor);
		y++;
		stoppingy += twoasquare;
		ellipseerror += ychange;
		ychange += twoasquare;
		if((2 * ellipseerror + xchange) > 0) {
			x--;
			stoppingx -= twobsquare;
			ellipseerror += xchange;
			xchange += twobsquare;
		}
	}

	x = 0;
	y = ry;
	xchange = ry * ry;
	ychange = rx * rx * (1 - 2 * ry);
	ellipseerror = 0;
	stoppingx = 0;
	stoppingy = twoasquare * ry;

	while (stoppingx <= stoppingy) {
		aa_plot4ellipsepoints(context, x, y, cx, cy, color, fillcolor);
		x++;
		stoppingx += twobsquare;
		ellipseerror += xchange;
		xchange += twobsquare;
		if((2 * ellipseerror + ychange) > 0) {
			y--;
			stoppingy -= twoasquare;
			ellipseerror += ychange;
			ychange += twoasquare;
		}

	}

}

/*
 * Draws 4 of the points on the ellipse
 * For internal use, Use aa_drawellipse instead.
 */
void aa_plot4ellipsepoints(aa_context *context, int x, int y,
                        int cx, int cy, int color, int fillcolor) {
	aa_putpixel(context, cx+x, cy+y, color);
	aa_putpixel(context, cx-x, cy+y, color);
	aa_putpixel(context, cx-x, cy-y, color);
	aa_putpixel(context, cx+x, cy-y, color);
	if(fillcolor != -1) {
		aa_drawline(context, cx-x+1, cy+y, cx+x-1, cy+y, fillcolor);
		aa_drawline(context, cx-x+1, cy-y, cx+x-1, cy-y, fillcolor);
	}

}


