/*
 * AAHelper - An AALib Helper Libary
 * (C) Gary Briggs 2003-4
 *
 * AAHelper is free software; you can redistribute it and/or modify
 * it under the  terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2.1 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT  ANY  WARRANTY;  without   even  the  implied  warranty  of
 * MERCHANTABILITY or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU
 * Library General Public License for more details.
 */ 

#include <aalib.h>
#include "aa_helper.h"
#include "aa_helperint.h"

/* x is 0 to 7, y is 0 to (font->height-1) */
/* Returns either 1 or 0 */
unsigned char getfontpixel(__AA_CONST struct aa_font *font, unsigned char c,
	unsigned char x, unsigned char y) {
    return (*(font->data + font->height * c + y) & (1<<x));
}


/* Pass this a context and a pointer to an allocated block of memory */
/* You can malloc it using:
	aa_context *context;
	unsigned char *frame;
	frame = (unsigned char *)malloc((size_t)(aa_scrwidth(context) * 9 *
			aa_scrheight(context) * (aa_currentfont(context)->height + 1)));
*/
						

void aa_renderframe(aa_context *c, unsigned char *frame) {
	__AA_CONST struct aa_font *font;
	__AA_CONST unsigned char *attrs;
	int x, y; /* x and y position in the final image buffer */
	int xchar, ychar; /* x and y position in the text */
	int xbit, ybit; /* x and y position in the individual letter */
	unsigned char pixel; /* getfontpixel's value */
	unsigned char attr, character; /* an aalib post-rendered char */
	int buf_pos;

	memset((void *)frame, 0x00, sizeof(unsigned char));

	for (xchar = 0; xchar < aa_scrwidth(c); xchar++) {
		for (ychar = 0; ychar < aa_scrheight(c); ychar++) {
			attrs = aa_attrs(c);
			attr = attrs[ychar * aa_scrwidth(c) + xchar];
			character = *(aa_text(c) + ychar * aa_scrwidth(c) + xchar);

			for (xbit = 0; xbit <= 7; xbit++) {
				for (ybit = 0; ybit <= font->height; ybit++) {
					x = xchar * 9 + (7 - xbit);
					y = ychar * (font->height+1) + ybit;
					buf_pos = aa_scrwidth(c) * y + x;
					pixel = getfontpixel(font, character, xbit, ybit);
					switch(attr) {
						case AA_NORMAL:
							if(pixel) frame[buf_pos] = 128;
							break;
						case AA_DIM:
							if(pixel) frame[buf_pos] = 64;
							break;
						case AA_BOLD:
							if(pixel) frame[buf_pos] = 196;
							break;
						case AA_BOLDFONT:
							if(pixel) frame[buf_pos] = 196;
							break;
						case AA_REVERSE:
							if(pixel == 0) frame[buf_pos] = 128;
							break;
						case AA_SPECIAL:
							if(pixel) frame[buf_pos] = 255;
							break;
						default:
							break;
					}
				}
			}
		}
	}
	return;
}
