/*
 * AAHelper - An AALib Helper Libary
 * (C) Gary Briggs 2003-4
 *
 * AAHelper is free software; you can redistribute it and/or modify
 * it under the  terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2.1 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT  ANY  WARRANTY;  without   even  the  implied  warranty  of
 * MERCHANTABILITY or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU
 * Library General Public License for more details.
 */ 

#include <aalib.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <aalib.h>

#include "aa_helper.h"

struct aa_point polygon[8];
aa_context *context;
aa_renderparams *params;

void render_and_wait_for_keypress(char *message);
void drawlines(void);
void drawcircles(void);
void drawfilledcircles(void);
void drawellipses(void);
void drawfilledellipses(void);
void drawconvexpoly(void);
void drawnonconvexpoly(void);
void drawcomplexpoly(void);

int main(int argc, char **argv) {

	/* INITIALISE */

	aa_parseoptions(NULL, NULL, &argc, argv);

	context = aa_autoinit(&aa_defparams);
	if (context == NULL) {
		printf("Failed to initialize aalib\n");
		exit(1);
	}
	aa_autoinitkbd(context, 0);
	params = aa_getrenderparams();
	aa_hidecursor(context);

	memset(context->imagebuffer,0,
			(size_t)(aa_imgwidth(context)*aa_imgheight(context)));


	drawlines();
	render_and_wait_for_keypress("Horizontal & Vertical Lines");

	drawcircles();
	render_and_wait_for_keypress("Empty Circles");

	drawfilledcircles();
	render_and_wait_for_keypress("Filled Circles");

	drawellipses();
	render_and_wait_for_keypress("Empty Ellipses");

	drawfilledellipses();
	render_and_wait_for_keypress("Filled Ellipses");

	drawconvexpoly();
	render_and_wait_for_keypress("Convex Polygon");

	drawnonconvexpoly();
	render_and_wait_for_keypress("NonConvex Polygon");

	drawcomplexpoly();
	render_and_wait_for_keypress("Complex Polygon");

	/* Shutdown */

	aa_close(context);
	return 0;
}

void render_and_wait_for_keypress(char *message) {
	aa_render(context, params, 0, 0,
		aa_imgwidth(context), aa_imgheight(context));

	aa_puts(context, 1, 1, AA_SPECIAL,
		 message);
	aa_puts(context, 1, 3, AA_SPECIAL,
		"Press any key to continue...");

	aa_flush(context);
	aa_getkey(context,1);

	memset(context->imagebuffer,0,
			(size_t)(aa_imgwidth(context)*aa_imgheight(context)));

}

void drawlines() {
	int i;
	for (i=0; i<aa_imgheight(context); i+=10) {
		aa_drawline(context,0,i,
				aa_imgwidth(context),i,255);
	}

	for (i=0; i<aa_imgwidth(context); i+=10) {
		aa_drawline(context,i,0,
				i, aa_imgheight(context),255);
	}
}

void drawcircles(void) {
	int i;
	int unit_width, unit_height;
	unit_width = aa_imgwidth(context)/12;
	unit_height = aa_imgheight(context)/12;

	for(i=2; i<10; i++) {
		aa_drawcircle(context, i*unit_width,
				i*unit_height, min(unit_width,unit_height),
				255, -1);
	}
}

void drawfilledcircles(void) {
	int i;
	int unit_width, unit_height;
	unit_width = aa_imgwidth(context)/12;
	unit_height = aa_imgheight(context)/12;

	for(i=2; i<10; i++) {
		aa_drawcircle(context, i*unit_width,
				i*unit_height, min(unit_width,unit_height),
				255, (i-1)*255/9);
	}
}

void drawellipses(void) {
	int i;
	int unit_width, unit_height;
	unit_width = aa_imgwidth(context)/12;
	unit_height = aa_imgheight(context)/12;

	for(i=1; i<9; i++) {
		aa_drawellipse(context, i*unit_width,
				i*unit_height, i*unit_width/2,
				i*unit_height/2,
				255, -1);
	}
}

void drawfilledellipses(void) {
	int i;
	int unit_width, unit_height;
	unit_width = aa_imgwidth(context)/12;
	unit_height = aa_imgheight(context)/12;

	for(i=1; i<9; i++) {
		aa_drawellipse(context, i*unit_width,
				i*unit_height, i*unit_width/2,
				i*unit_height/2,
				255, i*255/8);
	}
}

void drawconvexpoly(void) {
	int unit_width, unit_height;
	unit_width = aa_imgwidth(context)/5;
	unit_height = aa_imgheight(context)/5;

	polygon[5].x = unit_width * 1;
	polygon[5].y = unit_height * 3;

	polygon[4].x = unit_width * 2;
	polygon[4].y = unit_height * 4;

	polygon[3].x = unit_width * 3;
	polygon[3].y = unit_height * 3;

	polygon[2].x = unit_width * 3;
	polygon[2].y = unit_height * 2;

	polygon[1].x = unit_width * 2;
	polygon[1].y = unit_height * 1;

	polygon[0].x = unit_width * 1;
	polygon[0].y = unit_height * 2;

	aa_drawpoly(context, 6, polygon,  255, AA_HINT_CONVEX);
}

void drawnonconvexpoly(void) {
	int unit_width, unit_height;
	unit_width = aa_imgwidth(context)/8;
	unit_height = aa_imgheight(context)/8;

	polygon[7].x = unit_width * 3;
	polygon[7].y = unit_height * 4;

	polygon[6].x = unit_width * 1;
	polygon[6].y = unit_height * 7;

	polygon[5].x = unit_width * 4;
	polygon[5].y = unit_height * 5;

	polygon[4].x = unit_width * 7;
	polygon[4].y = unit_height * 7;

	polygon[3].x = unit_width * 5;
	polygon[3].y = unit_height * 4;

	polygon[2].x = unit_width * 7;
	polygon[2].y = unit_height * 1;

	polygon[1].x = unit_width * 4;
	polygon[1].y = unit_height * 3;

	polygon[0].x = unit_width * 1;
	polygon[0].y = unit_height * 1;

	aa_drawpoly(context, 8, polygon,  255, AA_HINT_NONCONVEX);
}

void drawcomplexpoly(void) {
	int unit_width, unit_height;
	unit_width = aa_imgwidth(context)/6;
	unit_height = aa_imgheight(context)/5;

	polygon[4].x = unit_width * 3;
	polygon[4].y = unit_height * 4;

	polygon[3].x = unit_width * 4;
	polygon[3].y = unit_height * 1;

	polygon[2].x = unit_width * 1;
	polygon[2].y = unit_height * 3;

	polygon[1].x = unit_width * 5;
	polygon[1].y = unit_height * 3;

	polygon[0].x = unit_width * 2;
	polygon[0].y = unit_height * 1;

	aa_drawpoly(context, 5, polygon,  255, AA_HINT_COMPLEX);

	aa_render(context, params, 0, 0,
		aa_imgwidth(context), aa_imgheight(context));
}

